/**
 * 客户端状态管理
 */
import { defineStore } from 'pinia'
import { ref, computed } from 'vue'
import { clientsApi } from '@/api/clients'
import { ElMessage } from 'element-plus'

export const useClientsStore = defineStore('clients', () => {
  // 状态
  const clients = ref([])
  const loading = ref(false)

  // 计算属性
  const enabledClients = computed(() => 
    clients.value.filter(client => client.enabled)
  )

  const connectedClients = computed(() => 
    clients.value.filter(client => client.connected)
  )

  const clientsCount = computed(() => clients.value.length)

  // 操作方法
  const fetchClients = async () => {
    loading.value = true
    try {
      const response = await clientsApi.getClients()
      if (response.success) {
        clients.value = response.data
      }
    } catch (error) {
      ElMessage.error(`获取客户端列表失败: ${error.message}`)
    } finally {
      loading.value = false
    }
  }

  const addClient = async (clientData) => {
    try {
      const response = await clientsApi.addClient(clientData)
      if (response.success) {
        clients.value.push(response.data)
        ElMessage.success('客户端添加成功')
        return response.data
      }
    } catch (error) {
      ElMessage.error(`添加客户端失败: ${error.message}`)
      throw error
    }
  }

  const updateClient = async (clientId, clientData) => {
    try {
      const response = await clientsApi.updateClient(clientId, clientData)
      if (response.success) {
        const index = clients.value.findIndex(c => c.id === clientId)
        if (index !== -1) {
          clients.value[index] = response.data
        }
        ElMessage.success('客户端更新成功')
        return response.data
      }
    } catch (error) {
      ElMessage.error(`更新客户端失败: ${error.message}`)
      throw error
    }
  }

  const deleteClient = async (clientId) => {
    try {
      const response = await clientsApi.deleteClient(clientId)
      if (response.success) {
        clients.value = clients.value.filter(c => c.id !== clientId)
        ElMessage.success('客户端删除成功')
      }
    } catch (error) {
      ElMessage.error(`删除客户端失败: ${error.message}`)
      throw error
    }
  }

  const testConnection = async (clientId) => {
    try {
      const response = await clientsApi.testConnection(clientId)
      if (response.success) {
        ElMessage.success('连接测试成功')
        // 更新客户端连接状态
        const client = clients.value.find(c => c.id === clientId)
        if (client) {
          client.connected = true
          client.version = response.version
          client.web_api_version = response.web_api_version
        }
      } else {
        ElMessage.error(`连接测试失败: ${response.error}`)
      }
      return response
    } catch (error) {
      ElMessage.error(`连接测试失败: ${error.message}`)
      throw error
    }
  }

  const getClientById = (clientId) => {
    return clients.value.find(c => c.id === clientId)
  }

  return {
    // 状态
    clients,
    loading,
    
    // 计算属性
    enabledClients,
    connectedClients,
    clientsCount,
    
    // 方法
    fetchClients,
    addClient,
    updateClient,
    deleteClient,
    testConnection,
    getClientById
  }
})
