/**
 * 种子状态管理
 */
import { defineStore } from 'pinia'
import { ref, computed } from 'vue'
import { torrentsApi } from '@/api/torrents'
import { ElMessage } from 'element-plus'

export const useTorrentsStore = defineStore('torrents', () => {
  // 状态
  const torrents = ref([])
  const globalStats = ref({
    download_speed: 0,
    upload_speed: 0,
    total_torrents: 0,
    active_torrents: 0,
    downloading: 0,
    seeding: 0,
    paused: 0
  })
  const clientsStatus = ref([])
  const loading = ref(false)
  const refreshing = ref(false) // 区分初始加载和刷新
  const operationLoading = ref(false) // 操作加载状态
  const selectedTorrents = ref([])
  const filterOptions = ref({
    status: 'all',
    clientIds: []
  })
  const lastUpdateTime = ref(null)

  // 计算属性
  const filteredTorrents = computed(() => {
    let filtered = torrents.value

    // 按状态过滤
    if (filterOptions.value.status !== 'all') {
      filtered = filtered.filter(torrent => {
        switch (filterOptions.value.status) {
          case 'downloading':
            return ['downloading', 'stalledDL', 'metaDL'].includes(torrent.state)
          case 'seeding':
            return ['uploading', 'stalledUP'].includes(torrent.state)
          case 'paused':
            return torrent.state.toLowerCase().includes('paused')
          case 'completed':
            return torrent.progress === 1
          default:
            return true
        }
      })
    }

    // 按客户端过滤
    if (filterOptions.value.clientIds.length > 0) {
      filtered = filtered.filter(torrent => 
        filterOptions.value.clientIds.includes(torrent.client_id)
      )
    }

    return filtered
  })

  const downloadSpeed = computed(() => globalStats.value.download_speed)
  const uploadSpeed = computed(() => globalStats.value.upload_speed)
  const totalTorrents = computed(() => globalStats.value.total_torrents)

  // 操作方法
  const fetchTorrents = async (clientIds = [], isRefresh = false) => {
    const loadingRef = isRefresh ? refreshing : loading
    loadingRef.value = true

    try {
      const response = await torrentsApi.getMainData(clientIds)
      if (response.success) {
        torrents.value = response.data.torrents
        globalStats.value = response.data.global_stats
        clientsStatus.value = response.data.clients_status
        lastUpdateTime.value = new Date()
      }
    } catch (error) {
      ElMessage.error(`获取种子列表失败: ${error.message}`)
    } finally {
      loadingRef.value = false
    }
  }

  // 静默刷新数据（不显示加载状态）
  const silentRefresh = async (clientIds = []) => {
    try {
      const response = await torrentsApi.getMainData(clientIds)
      if (response.success) {
        torrents.value = response.data.torrents
        globalStats.value = response.data.global_stats
        clientsStatus.value = response.data.clients_status
        lastUpdateTime.value = new Date()
      }
    } catch (error) {
      // 静默失败，不显示错误消息
      console.error('Silent refresh failed:', error)
    }
  }

  const pauseTorrents = async (hashes, clientId = null) => {
    operationLoading.value = true
    try {
      const response = await torrentsApi.pauseTorrents(hashes, clientId)
      if (response.success) {
        ElMessage.success('种子暂停成功')
        // 静默刷新数据，不显示加载状态
        await silentRefresh(filterOptions.value.clientIds)
      }
    } catch (error) {
      ElMessage.error(`暂停种子失败: ${error.message}`)
      throw error
    } finally {
      operationLoading.value = false
    }
  }

  const resumeTorrents = async (hashes, clientId = null) => {
    operationLoading.value = true
    try {
      const response = await torrentsApi.resumeTorrents(hashes, clientId)
      if (response.success) {
        ElMessage.success('种子恢复成功')
        // 静默刷新数据，不显示加载状态
        await silentRefresh(filterOptions.value.clientIds)
      }
    } catch (error) {
      ElMessage.error(`恢复种子失败: ${error.message}`)
      throw error
    } finally {
      operationLoading.value = false
    }
  }

  const deleteTorrents = async (hashes, deleteFiles = false, clientId = null) => {
    operationLoading.value = true
    try {
      const response = await torrentsApi.deleteTorrents(hashes, deleteFiles, clientId)
      if (response.success) {
        ElMessage.success('种子删除成功')
        // 静默刷新数据，不显示加载状态
        await silentRefresh(filterOptions.value.clientIds)
      }
    } catch (error) {
      ElMessage.error(`删除种子失败: ${error.message}`)
      throw error
    } finally {
      operationLoading.value = false
    }
  }

  const getTorrentDetails = async (torrentHash, clientId) => {
    try {
      const response = await torrentsApi.getTorrentDetails(torrentHash, clientId)
      if (response.success) {
        return response.data
      }
    } catch (error) {
      ElMessage.error(`获取种子详情失败: ${error.message}`)
      throw error
    }
  }

  const setSelectedTorrents = (torrents) => {
    selectedTorrents.value = torrents
  }

  const setFilter = (filter) => {
    filterOptions.value = { ...filterOptions.value, ...filter }
  }

  const formatSpeed = (speed) => {
    if (speed === 0) return '0 B/s'
    const units = ['B/s', 'KB/s', 'MB/s', 'GB/s']
    let unitIndex = 0
    while (speed >= 1024 && unitIndex < units.length - 1) {
      speed /= 1024
      unitIndex++
    }
    return `${speed.toFixed(1)} ${units[unitIndex]}`
  }

  const formatSize = (size) => {
    if (size === 0) return '0 B'
    const units = ['B', 'KB', 'MB', 'GB', 'TB']
    let unitIndex = 0
    while (size >= 1024 && unitIndex < units.length - 1) {
      size /= 1024
      unitIndex++
    }
    return `${size.toFixed(1)} ${units[unitIndex]}`
  }

  return {
    // 状态
    torrents,
    globalStats,
    clientsStatus,
    loading,
    refreshing,
    operationLoading,
    selectedTorrents,
    filterOptions,
    lastUpdateTime,

    // 计算属性
    filteredTorrents,
    downloadSpeed,
    uploadSpeed,
    totalTorrents,

    // 方法
    fetchTorrents,
    silentRefresh,
    pauseTorrents,
    resumeTorrents,
    deleteTorrents,
    getTorrentDetails,
    setSelectedTorrents,
    setFilter,
    formatSpeed,
    formatSize
  }
})
